/* dm.c:
 *	Just dump the byte-by-byte content of a file to the console...
 *  (like the 'dm' command in uMon)
 */
#ifndef BUILD_WITH_VCC
#define _FILE_OFFSET_BITS 64	// Need this for very big files
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef BUILD_WITH_VCC
extern int optind;
#else
#include <termios.h>
#include <unistd.h>
#endif
#include "version.h"
#include "utils.h"

#ifndef O_BINARY
#define O_BINARY 0
#endif

int	debug;
typedef unsigned char uchar;

char *usage_txt[] = {
	"Usage: dm [options] {filename} [size]",
	" Options:",
	" -o {offset}   start at offset from beginning",
	" -V            display version of tool",
	0
};

void
prascii(uchar *data,int cnt)
{
    int i;

    for(i=0;i<cnt;i++) {
        if ((*data > 0x1f) && (*data < 0x7f))
            printf("%c",*data);
        else
            putchar('.');
        data++;
    }
}

void
printMem(long addr, uchar *buf, int size)
{
	int i, col, showascii = 1;
	uchar *cp, *cp1;

	cp = cp1 = buf;
	printf("%08lx:  ",addr);
	for(col=1,i=0;i<size;i++,col++) {
		printf("%02x ",*cp++);
		if ((col == 8) || (col == 16)) {
			printf("  ");
			if (col == 16) {
				col = 0;
				if (showascii)
					prascii(cp1,16);
				cp1 += 16;
				addr += 16;
				if ((i+1) <size)
					printf("\n%08lx:  ",addr);
			}
		}
	}
	if ((showascii) && (col > 1)) {
		int	space;

		space = (3 * (17 - col)) + (col <= 8 ? 4 : 2);
		while(space--)
			putchar(' ');
		prascii(cp1,col-1);
	}
	printf("\n");
	return;
}

int fd;
struct termios orig;

void
fixstdin(int siglvl)
{
	if (siglvl != 0)
		fprintf(stderr,"<interrupt %d>\n",siglvl);
	tcsetattr(STDIN_FILENO,TCSANOW,&orig);
	if (siglvl != 0) {
		close(fd);
		exit(1);
	}
}

int
main(int argc,char *argv[])
{
	struct termios tmp;
	long 	addr;
	char	*fname, *buf, c;
	long	opt, size, offset;

	offset = -1;
	while ((opt=getopt(argc,argv,"o:V")) != EOF) {
		switch(opt) {
		case 'V':
			showVersion();
			break;
		case 'o':
			offset = strtol(optarg,0,0);
			break;
		default:
			usage(0);
		}
	}

	if (argc < optind+1)
		usage(0);

	fname = argv[optind];
	if (argc == optind+2)
		size = atoi(argv[optind+1]);
	else
		size = 256;

	buf = Malloc(size);	
	if ((fd = open(fname,O_RDONLY | O_BINARY))==-1) {
		perror("can't open file");
		exit(1);
	}

	if (offset > 0) {
		printf("Offset: %ld\n",offset);
		if (lseek(fd,offset,SEEK_SET) != offset)
			perror ("lseek:");
	}

	tcgetattr(STDIN_FILENO,&orig);
	signal(SIGINT, fixstdin);
	tcgetattr(STDIN_FILENO,&tmp);
	tmp.c_lflag &= ~ICANON;
	tmp.c_lflag &= ~ECHO;
	tcsetattr(STDIN_FILENO,TCSANOW,&tmp);

	addr = 0;
    while(1) {
        int lnsz, rtot;
        rtot = Read(fd,buf,size);
        if (rtot <= 0) {
            printf("\nEOF reached\n");
            break;
        }
        printMem(addr,buf,rtot);
        if (rtot < size) {
            printf("\nEOF reached.\n");
            break;
        }

        addr += size;
        lnsz = printf("more (y or n)?");
        fflush(stdout);
        c = getchar();
        while(lnsz) {
            putchar('\b');
            lnsz--;
        }
        fflush(stdout);
        if (c == 'n')
            break;
    }
	
	fixstdin(0);
	printf("\n");
	close(fd);
	exit(0);
}
